#include <math.h>
#include "Node.h"

Node::Node() : next(nullptr), name(""), state(""), row(0), col(0)
{
}

Node::Node(Coordinate coordinate) : next(nullptr), name(""), state("")
{
    convert_coordinate(coordinate);
}

Node::Node(City city)
    : next(nullptr), name(city.get_name()), state(city.get_state())
{
    convert_coordinate(city.get_coordinate());
}

Node::~Node()
{
}

Node  *Node::get_next()  const { return next; }
int    Node::get_row()   const { return row; }
int    Node::get_col()   const { return col; }
string Node::get_name()  const { return name; }
string Node::get_state() const { return state; }

void Node::convert_coordinate(const Coordinate& coordinate)
{
    row = round(2*(Coordinate::MAX_LATITUDE - coordinate.get_latitude()));
    col = round(2*(coordinate.get_longitude() - Coordinate::MIN_LONGITUDE));
}

bool Node::operator >(const Node &other)
{
    // This node is greater than the other node if and only if
    // this node's row is greater than the other node's row,
    // or, if the two rows are equal, this node's column is greater
    // than the other node's column.
    return    (row > other.row)
           || ((row == other.row) && (col > other.col));
}

ostream& operator <<(ostream& outs, const Node& node)
{
    // City node: Output a * followed by the city's name and state.
    if (node.name.length() > 0)
    {
        outs << "*" << node.name << " " << node.state;
    }

    // Boundary node: Output a #
    else
    {
        cout << "#";
    }

    return outs;
}
