#ifndef NODE_H_
#define NODE_H_

#include "City.h"
#include "Coordinate.h"

/**
 * A node of a linked list.
 */
class Node
{
public:
    /**
     * Default constructor.
     */
    Node();

    /**
     * Constructor.
     * @param coordinate the node's coordinate.
     */
    Node(Coordinate coordinate);

    /**
     * Constructor.
     * @param city the node's city.
     */
    Node(City city);

    /**
     * Destructor.
     */
    ~Node();

    /**
     * Getter.
     * @return the next pointer.
     */
    Node *get_next() const;

    /**
     * Getter.
     * @return the row.
     */
    int get_row() const;

    /**
     * Getter.
     * @return the column.
     */
    int get_col() const;

    /**
     * Getter.
     * @return the city's name.
     */
    string get_name() const;

    /**
     * Getter.
     * @return the city's state.
     */
    string get_state() const;

    Node *next;  // pointer to the next node in the list

    /**
     * Overloaded > operator to compare two nodes.
     */
    bool operator >(const Node& other);

    /**
     * Overloaded stream output insertion operator
     * to output a node to the output stream.
     * @param outs the output stream.
     * @param node the node to output.
     * @return the output stream.
     */
    friend ostream& operator <<(ostream& outs, const Node& node);

private:
    string name;   // city name,  or the empty string for a border location
    string state;  // city state, or the empty string for a border location
    int row;       // print row
    int col;       // print column

    /**
     * Convert a coordinate to a row and a column.
     * @param coordinate the coordinate to convert.
     */
    void convert_coordinate(const Coordinate& coordinate);
};

#endif /* NODE_H_ */
